/*
 * Copyright (c) 2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/time.h>
#include <sys/types.h>
#include <pthread.h>
#include <signal.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <portLib.h>
#include <taskLib.h>
#include <errnoLib.h>

#include <mor/pscan.h>
#include <mor/harmonica.h>
#include <mor/relayboard.h>
#include <mor/morLib.h>

int done = 0;

void 
ctrlc(int sig)
{
	done++;
}

double vx = 0, vy = 0 , w = 0;

void *
morTask(void *param)
{
	MOR_STR *mor = (MOR_STR *)param;
	struct timeval tv;
#ifdef VERBOSE
	int i;
#endif

	while (!done) {
		gettimeofday(&tv, NULL);
		while (morEvalCanBuffer(mor, 0) == OK)
			;
#ifdef VERBOSE
		for (i = 0; i < MOR_N_WHEELS; i++) {
			harmonicaShow(mor->wheels[i].steer);
			harmonicaShow(mor->wheels[i].drive);
		}
#endif
		morMove(mor, vx, vy, w, 0.02);
		morSendSpeed(mor, 0.02);
		usleep(20000);
	}
	return NULL;
}

int 
main(int argc, char *argv[])
{
	MOR_STR *mor;
	char *name = "/dev/pcan24";
	char buffer[128];
	char *s;
	int n;

	mor = morInit(name, NULL, true, false);
	if (mor == NULL) 
		exit(2);
	printf("morInit done\n");
	if (morInitPlatform(mor) == ERROR) {
		printf("initPlatform failed\n");
		morEnd(mor);
		exit(2);
	}
	morHoming(mor);
	signal(SIGINT, ctrlc);
	signal(SIGHUP, ctrlc);

	taskSpawn2("tMor", 100, VX_FP_TASK, 65536, morTask, mor);


	while (!done) {
		do {
			printf("vx vy w : ");
			s = fgets(buffer, 128, stdin);
			if (s == NULL) {
				done++;
				break;
			}
			n = sscanf(buffer, "%lg %lg %lg", &vx, &vy, &w);

		} while (n != 3);
	}

	
	morStop(mor);
	while (morEvalCanBuffer(mor, 0) == OK)
		;
	morEnd(mor);
	exit(0);
}
